from .conv import * 
from .dist import *
from .gen import * 
from .sort import * 
from .util import *
from .wcag import * 

"""Colorways facilitates procedural color palette and gradient generation.

Description:

    This library aims to provide a wide variety of tools for creating,
    changing, and sorting sets of colors. It might be found useful by
    desighers, game developers, data visualizers, gneerative artists, and
    other color monkeys. 

    Its goal is to make it easy and fun to generate pleasing and/or
    interesting color combinations. This is not a comprehensive set of
    tools for color scientists; it isn't blazingly fast; and it doesn't
    aspire to support every color space under the sun (and certainly not
    under every CIE standard illuminant.)

    It does provide tools to convert between a handful of colorspaces,
    including RGB, HSL, HSV, HWB, and L*a*b*. It also provides functions
    for comparing colors (including several CIE distance mesures and
    WCAG color contrast), interpolating between colors, blending colors,
    randomizing colors, and sorting colors. Finally, it provides a
    selection of parameterized color palette generators.

Conventions:

    To the extent possible, this library treats a 'color" generically,
    as a list of three floats between 0 and 1 inclusive.** It is up to the
    programmer to remember (or decide) whether 

        color = [0.0, 1.0, 0.5] 
    
    is 'springgreen', 'red', 'maroon', (the CSS name for that color
    point in RGB, HSL, or HSV space respectively) or a middlish gray
    color (as that point would appear as an HWB color.) Throughout this library
    a generic color value like this is often referred to as a 'vec3'. 

    Support is provided for conversion to and from some popular external
    formats, such as lists of 3 bytes (i.e. integers in the range 0 to
    255 inclusive), hexadecimal strings, and CSS keywords.

    Similar to the convention of using generic vec3s for colors, a 'palette' 
    is most often meant simply as a list of vec3s.** Many of the 

    Many of the functions in this library, particularly some conversion 
    functions, will attempt to determine whether an argument is a vec3 or a 
    palette and perform the connversion on either the single color or the whole
    palette.

^^^^^^^^^^^^^^^^^

    ** Important note: some color spaces (e.g. L*a*b*) are represented by
    three floats that are outside the [0,1] range. While intepretting an
    RGB palette as HSL might give an interesting result, interpreting a
    L*a*b palette as HSL will likely result in an error. At times, the 
    term 'vec3' might be used where it can also refer to a L*a*b* value
    or another 3-element list of numbers. Similarly, the term 'palette'
    will sometimes be loosely used to describe a list of colors in
    non-vec3 formats.


    Modules:
        bezier  Functions for bezier curve palettes.
        blend   Functions for blending colors.
        conv    Conversions between color spaces and representations.
        css     CSS named colors.
        gen     Various palette and gradient generation functions.
        dist    Color distance measures. 
        p3      Utilities for Processing's Python mode
        palettes    Some predefined palettes.
        ryb     RYB color space conversions.
        spectra Various approximations of the visible spectrum.
        sort:   Routines for sorting palettes by various criteria. 
        theory  Functions for traditional color theory palettes. 
        vars    Some color variables. These differ from CSS keywords.
        util:   Utility functions.
        wcag:   Web Content Accessibility Guidelines color contrast.
        masterpalette: Provides the MasterPalette class and utilities.

"""





from itertools import product
from random import random, choice, gammavariate, normalvariate
from math import pi, cos 
from .util import clamp01, reflect, lspace, hxx_lspace, randmix_vec3, randvec3_partial


"""
This module provides functions for generating palettes.

Exports:
    wave(t, freq=1.0, phase=0.0, offset=0.5, amp=0.5, fn=cos):
    wave_palette(n, fpoa, fn=cos):
    random_wave_palette(n):
    random_fpoa():
    golden_ratio_hsl_palette(n, hue, sat, lum):
    theory_hsl_palette(n, o1, o2, ra1, ra2, ra3), 
    harmony_hsl_palette(n, o1, o2, ra1, ra2, ra3,
    random_offset_palette(n, vec3, rng, map01fn=clamp01):
    value_offset_palette(n, vec3, rng, map01fn=clamp01):
    random_uniform_palette(n):
    random_walk(val, mindist, maxdist):
    random_walk_reflection(val, mindist, maxdist):
    random_walk_palette(n, vec3, minimum, maximum):
    random_walk_reflection_palette(n, vec3, minimum, maximum):
    randmix_palette(n, base, weight):
    randhue_palette(n, base):
    randred_palette = randhue_palette
    randsat_palette(n, base):
    randgreen_palette = randsat_palette
    randlum_palette(n, base):
    randblue_palette = randlum_palette
    randval_palette = randlum_palette
    randsatlum_palette(n, base):
    randgreenblue_palette = randsatlum_palette
    randsatval_palette = randsatlum_palette
    randhuesat_palette(n, base):
    randredgreen_palette = randhuesat_palette
    randhuelum_palette(n, base):
    randredblue_palette = randhuelum_palette
    randhueval_palette = randhuelum_palette
    gradient_palette(colors, steps): 
    hue_gradient_palette(colors, steps, bigarc=False): 
    perms_palette(pal)
    combo_palette(pal):
"""

__all__ = [
    'wave',
    'wave_fpoa_color',
    'wave_palette',
    'random_wave_palette',
    'random_fpoa',
    'golden_ratio_hsl_palette',
    'theory_hsl_palette',
    'harmony_hsl_palette',
    'random_offset_palette',
    'value_offset_palette',
    'random_uniform_palette',
    'random_walk',
    'random_walk_reflection',
    'random_walk_palette',
    'random_walk_reflection_palette',
    'randmix_palette',

    # Channel 1 
    'randhue_palette',
    'randred_palette',

    # Channel 2
    'randsat_palette',
    'randgreen_palette',
    'randwhite_palette',

    # Channel 3
    'randlum_palette',
    'randblue_palette',
    'randval_palette',
    'randblack_palette',

    # Channels 2 and 3
    'randsatlum_palette',
    'randgreenblue_palette',
    'randsatval_palette',
    'randwhiteblack_palette',

    # Channels 1 and 2
    'randhuesat_palette',
    'randredgreen_palette',
    'randhuelum_palette',
    'randhuewhite_palette',

    # Channels 1 and 3
    'randredblue_palette',
    'randhueval_palette',
    'randhueblack_palette',

    'gradient_palette',
    'hue_gradient_palette',

    # permutations palette
    'perms_palette',
    'combo_palette'
    
    ]


### Palette generation functions



def wave(t, freq=1.0, phase=0.0, offset=0.5, amp=0.5, fn=cos):
    """Undocumented"""
    return offset + amp * fn(2*pi * t*freq + phase)


# originally had (the equivalent of) : 
#   o + a * fn(pi*freq*t + pi*phase)
# Corrected to: 
#   o + a * fn(2*pi*freq*t + phase)




def wave_fpoa_color(t, fpoa, fn=cos, map01fn=clamp01):
    """Undocumented"""
    if map01fn is None:
        map01fn = lambda x: x
    return [
        map01fn(wave(t, *[arg[0] for arg in fpoa], fn=fn)),
        map01fn(wave(t, *[arg[1] for arg in fpoa], fn=fn)),
        map01fn(wave(t, *[arg[2] for arg in fpoa], fn=fn))]



def wave_palette(n, fpoa, fn=cos):
    """
    Generates a palette with sinusoid color channels.

    Args:
        n       number of colors in palette
        fpoa    Parameter list. See below.
        fn      Optional. sin or cos. Defaults to cos.  

    The fpoa parameter list is a structure like this:
        [[ frequency_red, frequency_green, frequency_blue ], 
         [ phase_red,     phase_green,     phase_blue     ], 
         [ offset_red,    offset_green,    offset_blue    ], 
         [ amplitude_red, amplitude_green, amplitude_blue ]]

    The equation is:

        offset + amplitude

    See Also:
        random_fpoa()

    """
    palette=[]
    for i in range(n):
        palette.append(wave_fpoa_color(i/n, fpoa, fn, clamp01))
    return palette



def random_wave_palette(n):
    """Generates a wave palette from random frequency, phase, offset and 
       amplitude parameters.

       Args:
           n: The number of colors in the return palette. 

       Returns:
           A palette. 
    """
    return wave_palette(n, random_fpoa())



def random_fpoa():
    """Generates random Frequency, Phase, Offset, and Amplitude parameters."""
    #fpoa = [[1.0, 1.0, 1.0], # Frequency  f: 0 < f (favor near 1, > 0.5, < 4)
    #        [0.0, 0.0, 0.0], # Phase:     p: 0 <= p <= 1
    #        [0.5, 0.5, 0.5], # Offset:    o: -a < o < a+1
    #        [0.5, 0.5, 0.5]] # Amplitude: a: 0 < a (favor near .5)

    f = [0.5+gammavariate(1.5,.5) for _ in range(3)]
    p = [random() for _ in range(3)]
    a = [normalvariate(0.5,.09) for _ in range(3)]
    o = [-a[i] + normalvariate((2*a[i]+1)/2, (2*a[i]+1)/7) for i in range(3)]

    return [f, p, o, a]



def golden_ratio_hsl_palette(n, hue=random(), sat=.5, lum=.5):
    """Undocumented"""
    palette = []
    phi = 0.6180339887498948482
    #hue = random()
    for i in range(n):
        palette.append([ hue, sat, lum ])
        hue += phi
        hue %= 1.0
    return palette



def theory_hsl_palette(n, o1, o2, ra1, ra2, ra3, 
                        sat=0.5, satrange=0.5, lum=0.5, lumrange=0.5, 
                        randsat=False, randlum=False, refang=None):
    """Undocumented"""
    palette = []
    if refang is None:
        ref = random() * 360.0
    else:
        ref = refang
    for i in range(n):
        ang = i/n * (ra1 + ra2 + ra3) 
        if ang < ra1:
            ang -= (ra1/2.0)
        elif ang < ra1 + ra2:
            ang += o1 - ra2
        else:
            ang += o2 - ra3
        new_sat, new_lum = sat, lum
        if randsat:
            new_sat = reflect(sat + (random() - 0.5) * satrange)
        if randlum:
            new_lum = reflect(lum + (random() - 0.5) * lumrange)
        palette.append( [((ref + ang)/360.0) % 1.0, new_sat, new_lum] )
    return palette



def harmony_hsl_palette(n, o1, o2, ra1, ra2, ra3,
                        sat=0.5, satrange=0.5, lum=0.5, lumrange=0.5, refang=None):
    """Undocumented"""
    palette = []
    if refang is None:
        ref = random() * 360.0
    else:
        ref = refang
    for i in range(n):
        rand = random() * (ra1 + ra2 + ra3)
        if rand < ra1:
            rand -= (ra1/2.0)
        elif rand < ra1 + ra2:
            rand += o1 - ra2
        else:
            rand += o2 - ra3
        new_sat = sat + (random() - 0.5) * satrange
        new_lum = lum + (random() - 0.5) * lumrange
        palette.append( [((ref + rand)/360.0) % 1.0, new_sat, new_lum] )
    return palette



def random_offset_palette(n, vec3, rng, map01fn=clamp01):
    """Undocumented"""
    if map01fn is None:
        map01fn = lambda x: x
    palette = [vec3]
    for i in range(n-1):
        new = list(vec3)
        new[0] = map01fn(new[0] + random() * 2 * rng - rng)
        new[1] = map01fn(new[1] + random() * 2 * rng - rng)
        new[2] = map01fn(new[2] + random() * 2 * rng - rng)
        palette.append(new)
    return palette



def value_offset_palette(n, vec3, rng, map01fn=clamp01):
    """Undocumented"""
    if map01fn is None:
        map01fn = lambda x: x
    palette = [vec3]
    for i in range(n-1):
        val = sum(vec3)/3
        new = val + 2*random() * rng - rng
        ratio = new/val
        palette.append([map01fn(c * ratio) for c in vec3])
    return palette



def random_uniform_palette(n):
    """Undocumented"""
    return random_offset_palette(n, [0.5, 0.5, 0.5], 0.5)



def random_walk(val, mindist, maxdist):
    """Undocumented"""
    return val + choice([-1, 1]) * (mindist + random()  * (maxdist - mindist))



def random_walk_reflection(val, mindist, maxdist):
    """Undocumented"""
    return reflect(val + choice([-1, 1]) * (mindist + random()  * (maxdist - mindist)))    



def random_walk_palette(n, vec3, minimum, maximum):
    """Undocumented"""
    palette = [vec3]
    #new = vec3
    for i in range(n-1):
        new = list(vec3)
        new[0] = random_walk(new[0], minimum, maximum) % 1
        new[1] = random_walk(new[1], minimum, maximum) % 1
        new[2] = random_walk(new[2], minimum, maximum) % 1
        palette.append(new)
    return palette



def random_walk_reflection_palette(n, vec3, minimum, maximum):
    """Undocumented"""
    palette = [vec3]
    #new = list(vec3)
    for i in range(n-1):
        new = list(vec3)
        new[0] = random_walk_reflection(new[0], minimum, maximum)
        new[1] = random_walk_reflection(new[1], minimum, maximum)
        new[2] = random_walk_reflection(new[2], minimum, maximum)
        palette.append(new)
    return palette



def randmix_palette(n, base, weight):
    """Undocumented"""
    palette = [base]
    for i in range(n-1):
        palette.append(randmix_vec3(base, weight))
    return palette



def randhue_palette(n, base):
    """Undocumented"""
    palette = [base]
    for _ in range(n-1):
        palette.append(randvec3_partial( [None, base[1], base[2]]))
    return palette

randred_palette = randhue_palette



def randsat_palette(n, base):
    """Undocumented"""
    palette = [base]
    for _ in range(n-1):
        palette.append(randvec3_partial([base[0], None, base[2]]))
    return palette

randgreen_palette = randsat_palette
randwhite_palette = randsat_palette


def randlum_palette(n, base):
    """Undocumented"""
    palette = [base]
    for _ in range(n-1):
        palette.append(randvec3_partial([base[0], base[1], None]))
    return palette

randblue_palette = randlum_palette
randval_palette = randlum_palette
randblack_palette = randlum_palette


def randsatlum_palette(n, base):
    """Undocumented"""
    palette = [base]
    for _ in range(n-1):
        palette.append(randvec3_partial([base[0], None, None]))
    return palette

randgreenblue_palette = randsatlum_palette
randsatval_palette = randsatlum_palette
randwhiteblack_palette = randsatlum_palette


def randhuesat_palette(n, base):
    """Undocumented"""
    palette = [base]
    for _ in range(n-1):
        palette.append(randvec3_partial([None, None, base[2]]))
    return palette

randredgreen_palette = randhuesat_palette
randhuewhite_palette = randhuesat_palette

def randhuelum_palette(n, base):
    """Undocumented"""
    palette = [base]
    for _ in range(n-1):
        palette.append(randvec3_partial([None, base[1], None]))
    return palette

randredblue_palette = randhuelum_palette
randhueval_palette = randhuelum_palette
randhueblack_palette = randhuelum_palette

# Linear interpolated (gradient) palettes

def gradient_palette(colors, steps): 
    """Undocumented"""
    if len(colors) < 2: 
        raise ValueError('gradient_palette: colors must be a list of length 2 or more.')
    if type(steps) != list:
        steps = [steps] * (len(colors)-1)
    ret = lspace(colors[0], colors[1], steps[0])
    for i in range(1, len(colors)-1):
        # Avoid duplicating colors by slicing off the first, but keep length 
        # as expected by adding one to the steps.
        ret += lspace(colors[i], colors[i+1], steps[i]+1)[1:]
    return ret



def hue_gradient_palette(colors, steps, bigarc=False): 
    """Undocumented"""
    if len(colors) < 2: 
        raise ValueError('gradient_palette: colors must be a list of length 2 or more.')
    if type(steps) != list:
        steps = [steps] * (len(colors)-1)
    ret = hxx_lspace(colors[0], colors[1], steps[0], bigarc)
    for i in range(1, len(colors)-1):
        # Avoid duplicating colors by slicing off the first, but keep length 
        # as expected by adding one to the steps.
        ret += hxx_lspace(colors[i], colors[i+1], steps[i]+1, bigarc)[1:]
    return ret

def perms_with_replace(n,k):
    return product(range(n), repeat=k)

def perms_palette(pal):
    return [[ pal[p[0]][0], 
              pal[p[1]][1], 
              pal[p[2]][2] ] for p in perms_with_replace(len(pal),3)]

def combo_palette(pal):
    if len(pal) > 1:
        vals = [ *pal[0], *pal[1] ]
    else:
        vals = [ *pal[0] ]
    return [[ vals[p[0]], 
              vals[p[1]], 
              vals[p[2]]  ] for p in perms_with_replace(len(vals),3)] 







from copy import copy, deepcopy
from .conv import hex2rgb, rgb2hsl, hsl2hsv, hsl2hwb, rgb2lab, norm2bytes
from .masterpalette import MasterPalette, new_order_by_dist_closure
from .dist import ciede2000

"""
This module provides CSS named colors.

Exports
    get_named_color(key):
    color_names_list():
    all_named_colors(get_as='hsl'):
    closest_named_color(hexcode, fn=ciede2000):
"""

# TODO: add str2rgb(str) for parsing css color strings. 


__all__ = [
    'get_named_color',
    'color_names_list',
    'all_named_colors',
    'closest_named_color'
]

# Lots of room for improvement in this module. Maybe use a customized
# MasterPalette object for easy access to sort by distance. 

_keyword_hexcode = {
    'aliceblue': '#F0F8FF',
    'antiquewhite': '#FAEBD7',
    'aqua': '#00FFFF',
    'aquamarine': '#7FFFD4',
    'azure': '#F0FFFF',
    'beige': '#F5F5DC',
    'bisque': '#FFE4C4',
    'black': '#000000',
    'blanchedalmond': '#FFEBCD',
    'blue': '#0000FF',
    'blueviolet': '#8A2BE2',
    'brown': '#A52A2A',
    'burlywood': '#DEB887',
    'cadetblue': '#5F9EA0',
    'chartreuse': '#7FFF00',
    'chocolate': '#D2691E',
    'coral': '#FF7F50',
    'cornflowerblue': '#6495ED',
    'cornsilk': '#FFF8DC',
    'crimson': '#DC143C',
    'cyan': '#00FFFF',
    'darkblue': '#00008B',
    'darkcyan': '#008B8B',
    'darkgoldenrod': '#B8860B',
    'darkgray': '#A9A9A9',
    'darkgreen': '#006400',
    'darkgrey': '#A9A9A9',
    'darkkhaki': '#BDB76B',
    'darkmagenta': '#8B008B',
    'darkolivegreen': '#556B2F',
    'darkorange': '#FF8C00',
    'darkorchid': '#9932CC',
    'darkred': '#8B0000',
    'darksalmon': '#E9967A',
    'darkseagreen': '#8FBC8F',
    'darkslateblue': '#483D8B',
    'darkslategray': '#2F4F4F',
    'darkslategrey': '#2F4F4F',
    'darkturquoise': '#00CED1',
    'darkviolet': '#9400D3',
    'deeppink': '#FF1493',
    'deepskyblue': '#00BFFF',
    'dimgray': '#696969',
    'dimgrey': '#696969',
    'dodgerblue': '#1E90FF',
    'firebrick': '#B22222',
    'floralwhite': '#FFFAF0',
    'forestgreen': '#228B22',
    'fuchsia': '#FF00FF',
    'gainsboro': '#DCDCDC',
    'ghostwhite': '#F8F8FF',
    'gold': '#FFD700',
    'goldenrod': '#DAA520',
    'gray': '#808080',
    'green': '#008000',
    'greenyellow': '#ADFF2F',
    'grey': '#808080',
    'honeydew': '#F0FFF0',
    'hotpink': '#FF69B4',
    'indianred': '#CD5C5C',
    'indigo': '#4B0082',
    'ivory': '#FFFFF0',
    'khaki': '#F0E68C',
    'lavender': '#E6E6FA',
    'lavenderblush': '#FFF0F5',
    'lawngreen': '#7CFC00',
    'lemonchiffon': '#FFFACD',
    'lightblue': '#ADD8E6',
    'lightcoral': '#F08080',
    'lightcyan': '#E0FFFF',
    'lightgoldenrodyellow': '#FAFAD2',
    'lightgray': '#D3D3D3',
    'lightgreen': '#90EE90',
    'lightgrey': '#D3D3D3',
    'lightpink': '#FFB6C1',
    'lightsalmon': '#FFA07A',
    'lightseagreen': '#20B2AA',
    'lightskyblue': '#87CEFA',
    'lightslategray': '#778899',
    'lightslategrey': '#778899',
    'lightsteelblue': '#B0C4DE',
    'lightyellow': '#FFFFE0',
    'lime': '#00FF00',
    'limegreen': '#32CD32',
    'linen': '#FAF0E6',
    'magenta': '#FF00FF',
    'maroon': '#800000',
    'mediumaquamarine': '#66CDAA',
    'mediumblue': '#0000CD',
    'mediumorchid': '#BA55D3',
    'mediumpurple': '#9370DB',
    'mediumseagreen': '#3CB371',
    'mediumslateblue': '#7B68EE',
    'mediumspringgreen': '#00FA9A',
    'mediumturquoise': '#48D1CC',
    'mediumvioletred': '#C71585',
    'midnightblue': '#191970',
    'mintcream': '#F5FFFA',
    'mistyrose': '#FFE4E1',
    'moccasin': '#FFE4B5',
    'navajowhite': '#FFDEAD',
    'navy': '#000080',
    'oldlace': '#FDF5E6',
    'olive': '#808000',
    'olivedrab': '#6B8E23',
    'orange': '#FFA500',
    'orangered': '#FF4500',
    'orchid': '#DA70D6',
    'palegoldenrod': '#EEE8AA',
    'palegreen': '#98FB98',
    'paleturquoise': '#AFEEEE',
    'palevioletred': '#DB7093',
    'papayawhip': '#FFEFD5',
    'peachpuff': '#FFDAB9',
    'peru': '#CD853F',
    'pink': '#FFC0CB',
    'plum': '#DDA0DD',
    'powderblue': '#B0E0E6',
    'purple': '#800080',
    'red': '#FF0000',
    'rosybrown': '#BC8F8F',
    'royalblue': '#4169E1',
    'saddlebrown': '#8B4513',
    'salmon': '#FA8072',
    'sandybrown': '#F4A460',
    'seagreen': '#2E8B57',
    'seashell': '#FFF5EE',
    'sienna': '#A0522D',
    'silver': '#C0C0C0',
    'skyblue': '#87CEEB',
    'slateblue': '#6A5ACD',
    'slategray': '#708090',
    'slategrey': '#708090',
    'snow': '#FFFAFA',
    'springgreen': '#00FF7F',
    'steelblue': '#4682B4',
    'tan': '#D2B48C',
    'teal': '#008080',
    'thistle': '#D8BFD8',
    'tomato': '#FF6347',
    'turquoise': '#40E0D0',
    'violet': '#EE82EE',
    'wheat': '#F5DEB3',
    'white': '#FFFFFF',
    'whitesmoke': '#F5F5F5',
    'yellow': '#FFFF00',
    'yellowgreen': '#9ACD32'
}


class colorinfo: pass

_string_lookup = {}
_color_lookup = {}

for key in _keyword_hexcode: 
    hexcode = _keyword_hexcode[key]
    rgb = hex2rgb(hexcode)
    rgb8 = tuple(norm2bytes(rgb))

    if rgb8 in _color_lookup:
        cdict = _color_lookup[rgb8]
        cdict.keys.append(key)
        _string_lookup[key] = cdict
        #_color_lookup[rgb8].keys.append(key)
        continue

    hsl = rgb2hsl(rgb)
    hsv = hsl2hsv(hsl)
    hwb = hsl2hwb(hsl)
    lab = rgb2lab(rgb)

    cdict = colorinfo() 
    cdict.keyword = key
    cdict.keys = [ key ]
    cdict.rgb8 = rgb8
    cdict.name = key
    cdict.key = key
    cdict.rgb = rgb
    cdict.hsl = hsl
    cdict.hsv = hsv
    cdict.hwb = hwb
    cdict.lab = lab
    cdict.hex = hexcode
    cdict.hexcode = hexcode

    _string_lookup[hexcode.lower()] = cdict
    _string_lookup[key] = cdict
    _color_lookup[rgb8] = cdict


def get_named_color(key): 
    """Returns the color with the given CSS name."""
    if type(key) is list:
        key = tuple(key)
    if type(key) is tuple: 
        if key in _color_lookup:
            return deepcopy(_color_lookup[key])
        else: 
            return None
    elif type(key) is str:
        key = key.lower()
        if key in _string_lookup:
            return deepcopy(_string_lookup[key])
        else: 
            return None
    else: 
        return None

   
def color_names_list():
    """Returns a sorted list of css color names."""
    return sorted(_keyword_hexcode.keys())


def all_named_colors(get_as='hsl'):
    """
    Returns a list of all css named colors in the requested color space.
    Default is hsl.
    """
    return [copy(ci.__dict__[get_as]) for ci in _color_lookup.values()]


_mp = MasterPalette(all_named_colors())
def closest_named_color(hexcode, fn=ciede2000):
    """
    Returns the css named color closest to the color given by hexcode. The
    distance function defaults to ciede2000.
    """
    _mp.create_ordering('__temp', new_order_by_dist_closure(hexcode, fn))
    return get_named_color(_mp.get_rgb8(order='__temp')[0])
    



from setuptools import setup
from setuptools import find_packages

# read the contents of your README file
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


setup(name='colorways',
    version='0.9.3',
    description='A Python library for procedural generation of color palettes, color manipulation, and color analysis.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/jeremymadea/colorways',
    author='Jeremy Madea',
    author_email='jdmadea@gmail.com',
    license='MIT',
    packages=find_packages(),
    package_data={"colorways": ["demos.zip"]},
    zip_safe=False)



from .internals import colpal_params_function, colpal_function
from .conv import rgb2hsl, hsl2rgb

"""This module provides functions for traditional color theory palettes."""

__all__ = [
    'complement',
    'hxx_complement',
    'hxx_triad',
    'hxx_analogous',
    'hxx_square',
    'hxx_complementary_pair',
    'split_palette',
    'tetradic_palette',
]

@colpal_function
def complement(base):
    """Returns the RGB complement of the color."""
    return [1.0-a for a in base]

@colpal_function
def hxx_complement(base):
    """Returns the Hxx complement of the color."""
    return [(base[0]+.5)%1, base[1], base[2]]

@colpal_function
def hxx_analogous(base):
    """Returns a 3-color analogous palette.""" 
    return [ 
        [(base[0] - (1/6.0))%1.0, base[1], base[2]],
        list(base),
        [(base[0] + (1/6.0))%1.0, base[1], base[2]]
    ]

@colpal_function
def hxx_triad(base):
    """Returns a triad palette."""
    return [ 
        [(base[0] - (1/3.0)) % 1.0, base[1], base[2]],
        list(base),
        [(base[0] - (2/3.0)) % 1.0, base[1], base[2]]
    ]
        

@colpal_function
def hxx_complementary_pair(base):
    """Returns a 2-color palette containing the base and its complement."""
    return [ list(base), [ (base[0] + 0.5) % 1.0, base[1], base[2] ]]



@colpal_function
def hxx_square(base):
    """
    Returns 4 color palette of the base color, and the base color with its 
    hue shifted 90, 180, and 270 degrees (or 0.25, 0.50, and 0.75 turns.) 
    """
    return [ list(base), 
             [ (base[0] + 0.25) % 1.0, base[1], base[2] ], 
             [ (base[0] + 0.50) % 1.0, base[1], base[2] ],
             [ (base[0] + 0.75) % 1.0, base[1], base[2] ]]



@colpal_params_function
def split_palette(base, angles):
    """
    Takes a base color or palette and a list of angles in [0, 1] turns.
    Base is assumed to be in an Hxx space. The last two channels are left 
    unchanged.
    """
    return [[(base[0]-a)%1.0, base[1], base[2]] for a in angles 
           ] + [list(base)] + [ 
            [(base[0]+a)%1.0, base[1], base[2]] for a in angles]


@colpal_params_function
def tetradic_palette(base, angle):
    """
    Takes a base color and an angle in [0, 1] turns. Base is assumed to be in
    Hxx space. A secondary base is specified by the angle from the base. The 
    returned palette consists of the base, its complement, the secondary base,
    and its complement. This scheme is otherwise known as a rectangular color 
    scheme.
    """
    return [
            list(base), 
            #rgb2hsl(complement(hsl2rgb(base))),  
            hxx_complement(base),  
            [(base[0]+angle)%1, base[1], base[2]], 
            [(base[0]+angle+.5)%1, base[1], base[2]] ]

